@@ -23,11 +23,11 @@ import (
 	"sync"
 	"time"
 
+	"github.com/alecthomas/kingpin/v2"
 	"github.com/go-kit/log"
 	"github.com/go-kit/log/level"
-	_ "github.com/go-sql-driver/mysql"
+	"github.com/go-sql-driver/mysql"
 	"github.com/prometheus/client_golang/prometheus"
-	"gopkg.in/alecthomas/kingpin.v2"
 )
 
 // Metric name parts.
@@ -62,9 +62,21 @@ var (
 	).Default("false").Bool()
 )
 
-// Metric descriptors.
+// metric definition
 var (
-	scrapeDurationDesc = prometheus.NewDesc(
+	mysqlUp = prometheus.NewDesc(
+		prometheus.BuildFQName(namespace, "", "up"),
+		"Whether the MySQL server is up.",
+		nil,
+		nil,
+	)
+	mysqlScrapeCollectorSuccess = prometheus.NewDesc(
+		prometheus.BuildFQName(namespace, exporter, "collector_success"),
+		"mysqld_exporter: Whether a collector succeeded.",
+		[]string{"collector"},
+		nil,
+	)
+	mysqlScrapeDurationSeconds = prometheus.NewDesc(
 		prometheus.BuildFQName(namespace, exporter, "collector_duration_seconds"),
 		"Collector time duration.",
 		[]string{"collector"}, nil,
@@ -80,11 +92,10 @@ type Exporter struct {
 	logger   log.Logger
 	dsn      string
 	scrapers []Scraper
-	metrics  Metrics
 }
 
 // New returns a new MySQL exporter for the provided DSN.
-func New(ctx context.Context, dsn string, metrics Metrics, scrapers []Scraper, logger log.Logger) *Exporter {
+func New(ctx context.Context, dsn string, scrapers []Scraper, logger log.Logger) *Exporter {
 	// Setup extra params for the DSN, default to having a lock timeout.
 	dsnParams := []string{fmt.Sprintf(timeoutParam, *exporterLockTimeout)}
 
@@ -104,38 +115,30 @@ func New(ctx context.Context, dsn string, metrics Metrics, scrapers []Scraper, l
 		logger:   logger,
 		dsn:      dsn,
 		scrapers: scrapers,
-		metrics:  metrics,
 	}
 }
 
 // Describe implements prometheus.Collector.
 func (e *Exporter) Describe(ch chan<- *prometheus.Desc) {
-	ch <- e.metrics.TotalScrapes.Desc()
-	ch <- e.metrics.Error.Desc()
-	e.metrics.ScrapeErrors.Describe(ch)
-	ch <- e.metrics.MySQLUp.Desc()
+	ch <- mysqlUp
+	ch <- mysqlScrapeDurationSeconds
+	ch <- mysqlScrapeCollectorSuccess
 }
 
 // Collect implements prometheus.Collector.
 func (e *Exporter) Collect(ch chan<- prometheus.Metric) {
-	e.scrape(e.ctx, ch)
-
-	ch <- e.metrics.TotalScrapes
-	ch <- e.metrics.Error
-	e.metrics.ScrapeErrors.Collect(ch)
-	ch <- e.metrics.MySQLUp
+	up := e.scrape(e.ctx, ch)
+	ch <- prometheus.MustNewConstMetric(mysqlUp, prometheus.GaugeValue, up)
 }
 
-func (e *Exporter) scrape(ctx context.Context, ch chan<- prometheus.Metric) {
-	e.metrics.TotalScrapes.Inc()
+// scrape collects metrics from the target, returns an up metric value.
+func (e *Exporter) scrape(ctx context.Context, ch chan<- prometheus.Metric) float64 {
 	var err error
-
 	scrapeTime := time.Now()
 	db, err := sql.Open("mysql", e.dsn)
 	if err != nil {
 		level.Error(e.logger).Log("msg", "Error opening connection to database", "err", err)
-		e.metrics.Error.Set(1)
-		return
+		return 0.0
 	}
 	defer db.Close()
 
@@ -147,15 +150,10 @@ func (e *Exporter) scrape(ctx context.Context, ch chan<- prometheus.Metric) {
 
 	if err := db.PingContext(ctx); err != nil {
 		level.Error(e.logger).Log("msg", "Error pinging mysqld", "err", err)
-		e.metrics.MySQLUp.Set(0)
-		e.metrics.Error.Set(1)
-		return
+		return 0.0
 	}
 
-	e.metrics.MySQLUp.Set(1)
-	e.metrics.Error.Set(0)
-
-	ch <- prometheus.MustNewConstMetric(scrapeDurationDesc, prometheus.GaugeValue, time.Since(scrapeTime).Seconds(), "connection")
+	ch <- prometheus.MustNewConstMetric(mysqlScrapeDurationSeconds, prometheus.GaugeValue, time.Since(scrapeTime).Seconds(), "connection")
 
 	version := getMySQLVersion(db, e.logger)
 	var wg sync.WaitGroup
@@ -170,14 +168,26 @@ func (e *Exporter) scrape(ctx context.Context, ch chan<- prometheus.Metric) {
 			defer wg.Done()
 			label := "collect." + scraper.Name()
 			scrapeTime := time.Now()
+			collectorSuccess := 1.0
 			if err := scraper.Scrape(ctx, db, ch, log.With(e.logger, "scraper", scraper.Name())); err != nil {
-				level.Error(e.logger).Log("msg", "Error from scraper", "scraper", scraper.Name(), "err", err)
-				e.metrics.ScrapeErrors.WithLabelValues(label).Inc()
-				e.metrics.Error.Set(1)
+				level.Error(e.logger).Log("msg", "Error from scraper", "scraper", scraper.Name(), "target", e.getTargetFromDsn(), "err", err)
+				collectorSuccess = 0.0
 			}
-			ch <- prometheus.MustNewConstMetric(scrapeDurationDesc, prometheus.GaugeValue, time.Since(scrapeTime).Seconds(), label)
+			ch <- prometheus.MustNewConstMetric(mysqlScrapeCollectorSuccess, prometheus.GaugeValue, collectorSuccess, label)
+			ch <- prometheus.MustNewConstMetric(mysqlScrapeDurationSeconds, prometheus.GaugeValue, time.Since(scrapeTime).Seconds(), label)
 		}(scraper)
 	}
+	return 1.0
+}
+
+func (e *Exporter) getTargetFromDsn() string {
+	// Get target from DSN.
+	dsnConfig, err := mysql.ParseDSN(e.dsn)
+	if err != nil {
+		level.Error(e.logger).Log("msg", "Error parsing DSN", "err", err)
+		return ""
+	}
+	return dsnConfig.Addr
 }
 
 func getMySQLVersion(db *sql.DB, logger log.Logger) float64 {
@@ -195,41 +205,3 @@ func getMySQLVersion(db *sql.DB, logger log.Logger) float64 {
 	}
 	return versionNum
 }
-
-// Metrics represents exporter metrics which values can be carried between http requests.
-type Metrics struct {
-	TotalScrapes prometheus.Counter
-	ScrapeErrors *prometheus.CounterVec
-	Error        prometheus.Gauge
-	MySQLUp      prometheus.Gauge
-}
-
-// NewMetrics creates new Metrics instance.
-func NewMetrics() Metrics {
-	subsystem := exporter
-	return Metrics{
-		TotalScrapes: prometheus.NewCounter(prometheus.CounterOpts{
-			Namespace: namespace,
-			Subsystem: subsystem,
-			Name:      "scrapes_total",
-			Help:      "Total number of times MySQL was scraped for metrics.",
-		}),
-		ScrapeErrors: prometheus.NewCounterVec(prometheus.CounterOpts{
-			Namespace: namespace,
-			Subsystem: subsystem,
-			Name:      "scrape_errors_total",
-			Help:      "Total number of times an error occurred scraping a MySQL.",
-		}, []string{"collector"}),
-		Error: prometheus.NewGauge(prometheus.GaugeOpts{
-			Namespace: namespace,
-			Subsystem: subsystem,
-			Name:      "last_scrape_error",
-			Help:      "Whether the last scrape of metrics from MySQL resulted in an error (1 for error, 0 for success).",
-		}),
-		MySQLUp: prometheus.NewGauge(prometheus.GaugeOpts{
-			Namespace: namespace,
-			Name:      "up",
-			Help:      "Whether the MySQL server is up.",
-		}),
-	}
-}
