@@ -21,9 +21,9 @@ import (
 	"fmt"
 	"strconv"
 
+	"github.com/alecthomas/kingpin/v2"
 	"github.com/go-kit/log"
 	"github.com/prometheus/client_golang/prometheus"
-	"gopkg.in/alecthomas/kingpin.v2"
 )
 
 const (
@@ -69,8 +69,10 @@ var (
 // This is mainly targeting pt-heartbeat, but will work with any heartbeat
 // implementation that writes to a table with two columns:
 // CREATE TABLE heartbeat (
-//  ts                    varchar(26) NOT NULL,
-//  server_id             int unsigned NOT NULL PRIMARY KEY,
+//
+//	ts                    varchar(26) NOT NULL,
+//	server_id             int unsigned NOT NULL PRIMARY KEY,
+//
 // );
 type ScrapeHeartbeat struct{}
 
