@@ -32,8 +32,8 @@ const infoSchemaAutoIncrementQuery = `
 		    when 'int'       then 31
 		    when 'bigint'    then 63
 		    end+(column_type like '% unsigned'))-1 as max_int
-		  FROM information_schema.tables t
-		  JOIN information_schema.columns c USING (table_schema,table_name)
+		  FROM information_schema.columns c
+		  STRAIGHT_JOIN information_schema.tables t USING (table_schema,table_name)
 		  WHERE c.extra = 'auto_increment' AND t.auto_increment IS NOT NULL
 		`
 
